#include "Classificador.h"
#include <opencv2/highgui/highgui.hpp>
#include <string>
#include <iostream>
#include "Obra.h"
#include "DadesMuseus.h"  
#include "Resultat.h"
#include <hash_map>

using namespace cv;
using namespace std;
#define NUM_FOLDS 3 

// constructors
Classificador::Classificador(void)
{   
	CDadesMuseus m_dm("",1);
}

Classificador::Classificador(string dbPath, int p_num_folds, int p_num_class_per_fold, 
	              int p_feature_type,  int p_num_veins, bool p_debug)
{   
	m_num_folds = p_num_folds;
	m_num_class_per_fold = p_num_class_per_fold;
	m_debug = p_debug;
    m_num_veins = p_num_veins;
	CDadesMuseus dm(dbPath, p_feature_type);
	dm.setLocalStoragePath(dbPath.append("/local"));
    dm.setLocalStorage(true);
	setDM(dm);    
}



// destructors
Classificador::~Classificador(void)
{
}
 
/// set one fold as test set and the other as trainning
void Classificador::setTestSet(int foldId) { 
	Obra obr;
	for ( int j = 1; j <= m_dm.getNum_elements(); j++) {  
				obr = m_dm.getObra(j); 
				if (obr.getFold() == foldId) {
					obr.setIsTrainning_set(false);
				} else {
					obr.setIsTrainning_set(true);
				}; 
				m_dm.setObra(j, obr);  
	}; 	
}

// executa el process de classificaci
void Classificador::run() {
	
	
	// inici de la classificacio: calcul dels folds
	spliIntoFolds(); 
	// if (m_debug) {showFolds();};
	showFolds();
		 
	int bondat = 0;
	for (int ii=1; ii<=getNumFolds(); ii++) {
		 
			// k-fold iter ii
			cout << "---------------------------------------------" << endl;
			cout << "Iteracio #"; cout << ii<< endl; 
			setTestSet(ii);
			//if (m_debug) {showTrainningAndTestSets();}; 
			showTrainningAndTestSets();  
			classifyTestSet();
			bondat += showResult(false);  
	}
	bondat = bondat/getNumFolds();
		 
	cout << "---------------------------------------------" << endl;
	cout << "El promig dels "; cout << getNumFolds() ; cout << " folds es: "; cout << bondat;   cout << " %."<< endl;
	cout << "---------------------------------------------" << endl; 
	 
}

/// show current training and test set 
void Classificador::showTrainningAndTestSets(void) { 
	string  fold1;
	string  fold2;
	char buffer [33]; 
	Obra obr;
	for ( int j = 1; j <= m_dm.getNum_elements(); j++) {  
				obr = m_dm.getObra(j); 
				_itoa_s(obr.getId(), buffer, 10);
				if (obr.getIsTrainning_set())  {
					fold1.append(buffer); fold1.append(", "); 
				} else {
					fold2.append(buffer); fold2.append(", "); 
				};    
	}; 
	cout << "---------------------------------------------" << endl;
	cout << "Conjunt d'entrenament: " << fold1 << endl;  
	cout << "Conjunt de Test: " << fold2 << endl;  
}

 
/// split into 3 folds
void Classificador::spliIntoFolds(void) {   
		int random_integer = 0;
		int _class = 0;
		Obra obr;
		for ( int j = 1; j <=  m_dm.getNum_elements(); j++) {    
		        obr = m_dm.getObra(j);   
				_class = obr.getClass(); 

				// numero aleatori entre 1 i m_num_folds 
				random_integer = 1 + rand()%m_num_folds;  
				while (! isValidFold(random_integer, _class))   {  
				   random_integer = 1 + rand()%m_num_folds;  
				};   

				// assignem el fold a la obra
				obr.setFold(random_integer); 
				m_dm.setObra(j, obr); 
		} 
} 


// this funtion determines if we can assign fold_num as the folder for given obra obra_id
// since we have the restriction that in each folder we should have one element of each class   
bool Classificador::isValidFold(int fold_num, int a_classID) {  
	if (fold_num == 0) {
	   return false; 
	}

	// we have to check if we already have an element with this class already assigned
	// in the same folder 
	/**
	int  current_class;
	int  current_fold;
	Obra obr;
	for ( int j = 1; j <=  m_dm.getNum_elements(); j++) { 
				obr = m_dm.getObra(j); 
				current_class = obr.getClass();
				current_fold = obr.getFold(); 
				if ((current_fold == fold_num) && (current_class == a_classID) ) { 
					return false; 
				} else { 
					// continue
				} 
	}; 
	**/

	// obtenim el nombre d'elements de la classe donada en el fold
	int  current_class;
	int  current_fold;
	int  num_elements=0;
	Obra obr; 
	for ( int j = 1; j <=  m_dm.getNum_elements(); j++) { 
				obr = m_dm.getObra(j); 
				current_class = obr.getClass();
				current_fold = obr.getFold(); 
				if ((current_fold == fold_num) && (current_class == a_classID) ) { 
					 num_elements++;
				} else { 
					// continue
				} 
	}; 

	if (num_elements  < getNumClass_per_fold()) {
		return true;  
	} else {
		return false;  
	}
}
	

/// mostra els elements dels m_num_folds folds que hem construit 
void Classificador::showFolds(void) {
	const int MAXFOLDS = 10; 
	string  folds[MAXFOLDS] = {"","","","","","","","","",""}; 
	int  current_fold = 0;
	Obra obr;
	char buffer [33];
	for ( int j = 1; j <=  m_dm.getNum_elements(); j++) {  
				obr = m_dm.getObra(j);   
				current_fold = obr.getFold(); 
				_itoa_s(obr.getId(), buffer, 10);
				folds[current_fold-1].append(buffer); 
				folds[current_fold-1].append(", ");  
	}; 

	cout << "---------------------------------------------" << endl;
	cout << "Conjunt de dades dividit en "; cout << m_num_folds ; cout << " folds." << endl; 

	for (int ii=0; ii<m_num_folds; ii++) {
		 
			// k-fold ii
			cout << "---------------------------------------------" << endl;
			cout << "Al Fold "; cout << ii+1 ; cout << " hi tenim: " << folds[ii] << endl; 
		}

	/**
	cout << "---------------------------------------------" << endl;
	cout << "Split dataset in 3 folds" << endl;
	cout << "---------------------------------------------" << endl;
	cout << "Fold 1 elements" << endl;
	cout << "---------------------------------------------" << endl;
	cout << fold1 << endl; 
	cout << "---------------------------------------------" << endl;
	cout << "Fold 2 elements" << endl;
	cout << "---------------------------------------------" << endl;
	cout << fold2 << endl; 
	cout << "---------------------------------------------" << endl;
	cout << "Fold 3 elements" << endl;
	cout << "---------------------------------------------" << endl;
	cout << fold3 << endl;
	cout << "---------------------------------------------" << endl;  
	**/
} 
  
 
// metode 1NN
void Classificador::classifyTestSet(void){
     
	// per tot el conjunt de test cal trobar el vei mes proper
	Obra  obr;
	Obra  vei_mes_proper;
	m_num_res = 0;
	for ( int j = 1; j <= m_dm.getNum_elements(); j++) {  
				obr = m_dm.getObra(j);  
				if (!obr.getIsTrainning_set()) {
				  
				    // vei mes proper
			 	    vei_mes_proper = getObraMesPropera(obr);  

					// afegim resultat 
					Resultat res(obr.getId(), obr.getClass(), vei_mes_proper.getClass() );
					res.set_obra_mes_semblant_id(vei_mes_proper.getId());
					m_num_res++;
					m_resultats[m_num_res]=res; 
				};    
	}; 

}

int Classificador::showResult(bool detallat){
    cout << "-----------------------------------------------" << endl;
	cout << "Resultat de la classificacio del conjunt de test:" << endl;

	Resultat res;  
	int numOK=0;
	int numNOK=0;
	for ( int j = 1; j <= m_num_res; j++) {   
		 res = m_resultats[j];
		 if (m_debug)  {
		 cout << "---------------------------------------------" << endl;
			 cout << "Obra:";				 cout << res.get_obra_id() << endl;
			 cout << "Obra mes semblant:";   cout << res.get_obra_mes_semblant_id() << endl;
			 cout << "Classe Real:";		 cout << res.get_known_class() << endl;
			 cout << "Classe Calculada:";	 cout << res.get_calculated_class() << endl;  
		 }
		 if (res.get_calculated_class() == res.get_known_class()) {
			 numOK++;
		 } else {
			 numNOK++;
		 }
	}; 
	int bondat = numOK * 100; 
	bondat = bondat / (numOK + numNOK);
	 
	cout << "Total Encerts: " ;	cout << numOK << endl;
	cout << "Total Errors : " ;	cout << numNOK << endl;
	cout << "Bondat  (%)  : " ;	cout << bondat << endl;

	return bondat;
} 

 void Classificador::setDM(CDadesMuseus dm){
 m_dm = dm;
 }

Obra Classificador::getObraMesPropera(Obra obr) {
    
 	// algorisme KNN 
	// trobem els K veins mes propers
	// escombrem el training set
    Obra  obr_ts;
	Obra  vei_mes_proper;
	map<int,int> obres_properes;  
	int k = m_num_veins; 
	map<int,int>::iterator it;
	string  llistaObres;
	char buffer [33]; 

	for ( int i = 1; i <= k; i++) {   
		int distancia_mes_propera = -1;
		int distancia = 0;
		int obra_selec = 0;
		for ( int j = 1; j <= m_dm.getNum_elements(); j++) {  
			 
			 //check obra not already selected
			 it = obres_properes.find(j);
             if(it == obres_properes.end()) {
					obr_ts = m_dm.getObra(j);  
					if (obr_ts.getIsTrainning_set()) {
				  
						// calculem la distancia i qens quedem amb el valor mes petit
						distancia = obr.distancia(obr_ts);
						if ((distancia < distancia_mes_propera) || (distancia_mes_propera == -1)) { 
							distancia_mes_propera = distancia;
							//obres_properes [j] = i;
							obra_selec = j;
						}
					};    
			 }
		}; 
		obres_properes [obra_selec] = i;
		_itoa_s(obra_selec, buffer, 10);
		llistaObres.append(buffer); llistaObres.append(", "); 
	}

	if (m_debug) {cout << "Les obres mes semblants a " << obr.getId() << " son: " << llistaObres << endl;  };

	// cerca classe mes repetida entre els conjunt d'obres properes
	int clase_mes_repetida = 0; int clase = 0;
	int count=0; int max_count=0;
	Obra  obr_ts1, obr_ts2;

	for ( int i = 1; i <=  m_dm.getNum_elements(); i++) { 
		obr_ts1 = m_dm.getObra(i);
		clase = obr_ts1.getClass();

		// ha d'estar en la llista
		it = obres_properes.find(i);
        if(it != obres_properes.end()) {
			
		    //_itoa_s(i, buffer, 10);
		    //llistaObres.append(buffer); llistaObres.append(", "); 

			count = 0;
			for ( int j = 1; j <= m_dm.getNum_elements(); j++) { 
				// ha d'estar en la llista
				it = obres_properes.find(j);
				if(it != obres_properes.end()) {
					obr_ts2 = m_dm.getObra(j);
					if (obr_ts2.getClass() == clase) {  
						count++; 
					}
				}
			}
			if (count > max_count) {
				max_count = count;
				clase_mes_repetida = clase;
				vei_mes_proper = obr_ts1;
			}
		}
	}
	return  vei_mes_proper;
}

void Classificador::showAllImages(void) {
	m_dm.showAllImages();
}

int Classificador::getNumFolds(void){
	return m_num_folds;
}

int Classificador::getNumClass_per_fold(void){
	return m_num_class_per_fold;
} 