#include "DadesMuseus.h" 
#include "Obra.h"
#include "DadesMuseusException.h"
#include <iostream>
#include <fstream>
#include <windows.h>
#include <winhttp.h>
#include <stdio.h>
#include <curl/curl.h>
#include <vector>
#include <sys/stat.h>

using namespace std;

ostream& operator<<(ostream& stream,vector<string> ob)
{
	stream << "[";
	for(vector<string>::iterator it=ob.begin();it!=ob.end();it++) {		
		if(it!=ob.begin()) {
			stream << "; ";
		}
		stream << "\"" << *it << "\"";	
	}
	stream << "]";

	return stream;
}

ostream& operator<<(ostream& stream,vector<int> ob)
{
	stream << "[";
	for(vector<int>::iterator it=ob.begin();it!=ob.end();++it) {
		if(it!=ob.begin()) {
			stream << "; ";
		}
		stream << *it;	
	}
	stream << "]";

	return stream;
}

CDadesMuseus::CDadesMuseus(void)  : m_Dictionaries(NULL), m_UseLocalStorage(false), m_LocalStoragePath("")
{
	//nothing
}

CDadesMuseus::CDadesMuseus(string dbPath, int p_feature_type)  : m_Dictionaries(NULL), m_UseLocalStorage(false), m_LocalStoragePath("")
{ 
	m_feature_type= p_feature_type;
	
		// Set the database path
	m_dbPath=dbPath;
	setLocalStoragePath(dbPath.append("/local"));
    setLocalStorage(true);

	// Load the dictionaries
	loadDictionaries();

	// Load image Labels
	loadImageLabels();

	// Load image URLs
	loadImageURLs();


}

CDadesMuseus::~CDadesMuseus(void)
{
}

size_t CDadesMuseus::WriteMemoryCallback(void *ptr, size_t size, size_t nmemb, void *data) {
    size_t realsize = size * nmemb;
    std::vector<char> *vec = (vector<char>*)data;
    for(size_t i = 0;i < realsize;i++)		
        vec->push_back(((char*)ptr)[i]);
    return realsize;
}

bool CDadesMuseus::downloadImage(string url,Mat &image) {
	CURL *curl=NULL;
	CURLcode res;
	bool retVal=false;	
	vector<char> buffer;

	// Initialize the cURL library object/structure
	curl = curl_easy_init(); 

	if(curl) {
		// Open the URL
        curl_easy_setopt(curl, CURLOPT_URL, url.c_str());

#ifdef _DEBUG
		// Show connection status
        curl_easy_setopt(curl, CURLOPT_VERBOSE, 1); // tell us what is happening
#endif //_DEBUF

        // Tell libcurl where to write the image
        curl_easy_setopt(curl,CURLOPT_WRITEFUNCTION, WriteMemoryCallback);
        curl_easy_setopt(curl,CURLOPT_WRITEDATA, (void *) &buffer);		
		
        // Get the image from the specified URL
        res = curl_easy_perform(curl);

		// decode memory buffer using OpenCV		
		if(CURLE_OK == res) {			
			image = imdecode(Mat(1, buffer.size(), CV_8UC1, buffer.data()), CV_LOAD_IMAGE_UNCHANGED);
			if(!image.empty())
				retVal=true;	
		}

        // always cleanup
        curl_easy_cleanup(curl);        
		buffer.clear();
  }

  return retVal;
}

void CDadesMuseus::loadDictionaries(void) {
	
	// Remove old dictionaries
	if(m_Dictionaries) {
		for(int i=0;i<DADES_MUSEUS_NUMLABELS;i++) {
			delete m_Dictionaries[i];
		}
	}

	// Allocate the new dictionary objects	
	m_Dictionaries=new CLabelDictionary*[DADES_MUSEUS_NUMLABELS];
	
	// Authors dictionary
	m_Dictionaries[LabelID::Author]=new CLabelDictionary("Author names");
	m_Dictionaries[LabelID::Author]->loadFile("authors.dict",m_dbPath);

	// Techniques dictionary
	m_Dictionaries[LabelID::Technique]=new CLabelDictionary("Techniques");
	m_Dictionaries[LabelID::Technique]->loadFile("techniques.dict",m_dbPath);

	// Forms dictionary
	m_Dictionaries[LabelID::Form]=new CLabelDictionary("Forms");
	m_Dictionaries[LabelID::Form]->loadFile("forms.dict",m_dbPath);

	// Types dictionary
	m_Dictionaries[LabelID::Type]=new CLabelDictionary("Types");
	m_Dictionaries[LabelID::Type]->loadFile("types.dict",m_dbPath);

	// Schools dictionary
	m_Dictionaries[LabelID::School]=new CLabelDictionary("Shools");
	m_Dictionaries[LabelID::School]->loadFile("schools.dict",m_dbPath);

	// Time Frames dictionary
	m_Dictionaries[LabelID::TimeFrame]=new CLabelDictionary("Time frames");
	m_Dictionaries[LabelID::TimeFrame]->loadFile("timeFrames.dict",m_dbPath);
}

void CDadesMuseus::loadImageLabels(void) {
	ifstream *inputFile=NULL;
	char line[1024];
	string fullFileName;
	string fileName="labels.txt";

	// Build the full filename of the URLs file
	if(m_dbPath.empty()) {
		fullFileName=fileName;		
	} else {		
		if(m_dbPath.at(m_dbPath.length()-1)=='/' || m_dbPath.at(m_dbPath.length()-1)=='\\') {
			fullFileName=m_dbPath+fileName;
		} else {
			fullFileName=m_dbPath+"/"+fileName;			
		}
	}
	
	// Open the file
	inputFile=new ifstream(fullFileName);

	// Parse the file
	if(inputFile->is_open())
	{
		int imageId;
		vector<int> vec(6);
		while (inputFile->good() )
		{
			// Skip comment lines
			inputFile->getline(line,1023);
			if(line[0]=='#')
				continue;
			
			// Parse the input data (ImageID Author Technique Form Type School TimeFrame)
			sscanf_s(line,"%d %d %d %d %d %d %d",&imageId, &vec[LabelID::Author],
				&vec[LabelID::Technique],&vec[LabelID::Form],&vec[LabelID::Type],
				&vec[LabelID::School],&vec[LabelID::TimeFrame]);			
			
			// Add the labels to the database
			m_ImageLabels[imageId]=vec; 
		}
		inputFile->close();	
    } else {	
		string msg("Cannot open Labels file \"");
		msg+=fullFileName;
		msg+="\".";
		throw CDadesMuseusException(msg);
	}    		
}

void CDadesMuseus::loadImageURLs(void) {
	ifstream *inputFile=NULL;
	char line[1024];
	string fullFileName;
	string fileName="url.txt";

	// Build the full filename of the URLs file
	if(m_dbPath.empty()) {
		fullFileName=fileName;		
	} else {		
		if(m_dbPath.at(m_dbPath.length()-1)=='/' || m_dbPath.at(m_dbPath.length()-1)=='\\') {
			fullFileName=m_dbPath+fileName;
		} else {
			fullFileName=m_dbPath+"/"+fileName;			
		}
	}

	// Open the file
	inputFile=new ifstream(fullFileName);

	// Parse the file
	
	if(inputFile->is_open())
	{
		int label;
		m_num_elements = 0;
		char desc[1024];
		while (inputFile->good() )
		{
			// Skip comment lines
			inputFile->getline(line,1023);
			if(line[0]=='#')
				continue;
			
			// Parse the input data
			sscanf_s(line,"%d %[^\r\n]",&label, desc,1023);
			
			// Adds the URL to the list
			string strDesc(desc);
			m_ImageURL[label]=strDesc;

			// creem un objecte de tipus obra d'art 
			Obra obr(strDesc, label, m_LocalStoragePath);

			if (!obr.getImage().empty()) { 
				// fen servir l'atribut author per classificar l'obra
				
				switch (getFeatureType()) {
					case(1):
						obr.setClass(m_ImageLabels[label][LabelID::Author]);  
						break;
					case(2):
						obr.setClass(m_ImageLabels[label][LabelID::Technique]);  
						break;
                    case(3):
						obr.setClass(m_ImageLabels[label][LabelID::Form]);  
						break;
					case(4):
						obr.setClass(m_ImageLabels[label][LabelID::Type]);  
						break;
					case(5):
						obr.setClass(m_ImageLabels[label][LabelID::School]);  
						break;
					case(6):
						obr.setClass(m_ImageLabels[label][LabelID::TimeFrame]);  
						break; 
				}
				
			
				// afegim l'obra a la llista d'obres d'art indexada per la primera columna a url.txt
				m_num_elements++;
				m_Obres[m_num_elements]=obr;   
			} else {
				cout << "Image not loaded:"  ; cout << strDesc << endl;
			}
		}
		inputFile->close();	

		// log
		cout << "---------------------------------------------" << endl;
		cout << "Nombre d'elements trobats:"  ; cout << m_num_elements << endl;  
    } else {	
		string msg("Cannot open URLs file \"");
		msg+=fullFileName;
		msg+="\".";
		throw CDadesMuseusException(msg);
	}    		
}

bool CDadesMuseus::getImage(int imagePos,Mat &image) {
	bool imageLoaded=false;
	string fullFileName;

	// Check the image position
	if(imagePos<0) {
		throw CDadesMuseusException("getImage:Image position is out of range");
	}
	if(m_ImageURL.count(imagePos)<1) {
		throw CDadesMuseusException("getImage:Image not found.");
	}
	
	// Try to read the image from the local folder
	imageLoaded=false;
	if(m_UseLocalStorage) {
		char fileName[1024];

		// Creat the filename
		sprintf_s(fileName,"Image_%07d.jpg",imagePos,1023);

		// Create the full filename
		if(m_LocalStoragePath.empty()) {
			fullFileName=string(fileName);
		} else {		
			if(m_LocalStoragePath.at(m_LocalStoragePath.length()-1)=='/' || m_LocalStoragePath.at(m_LocalStoragePath.length()-1)=='\\') {
				fullFileName=m_LocalStoragePath+string(fileName);
			} else {
				fullFileName=m_LocalStoragePath+"/"+string(fileName);			
			}
		}
		// Read the image from the local storage path
		image=imread(fullFileName);
		if(!image.empty()) {
			imageLoaded=true;
		}
	} 

	// Download the image from internet
	if(!imageLoaded) {
		// Download the image
		if(downloadImage(m_ImageURL[imagePos],image)) {
			imageLoaded=true;			
		}
		// Save the image
		if(imageLoaded && m_UseLocalStorage) {
			imwrite(fullFileName,image);
		}
	}

	return imageLoaded;
}

int CDadesMuseus::getNumImages(void) {
	return m_ImageURL.size();	
}

Mat CDadesMuseus::getLabelsMatrix(Mat *var_idx,Mat *sample_idx) {
	int numImages=0;
	int dimension=0;	

	// Get the output dimension
	if(var_idx) {
		if(var_idx->size().width>1 && var_idx->size().height>1)
			throw CDadesMuseusException("getLabelsMatrix:var_idx must be a row or column vector");
		dimension=max(var_idx->size().width,var_idx->size().height);
	} else {
		dimension=DADES_MUSEUS_NUMLABELS;
	}

	// Get the output dimension
	if(sample_idx) {
		if(sample_idx->size().width>1 && sample_idx->size().height>1)
			throw CDadesMuseusException("getLabelsMatrix:sample_idx must be a row or column vector");
		numImages=max(sample_idx->size().width,sample_idx->size().height);
	} else {
		numImages=getNumImages();
	}

	// Create the output matrix
	Mat labels=Mat(numImages,dimension,CV_32S);
	vector<int> imageLabels;
	int imgIdx,labIdx;
	for(int i=0;i<numImages;i++) {
		// Get the image position		
		if(sample_idx) {
			imgIdx=sample_idx->at<int>(i);
			getImageLabels(imgIdx,imageLabels);
		} else {
			getImageLabels(i+1,imageLabels);
		}

		// Add selected labels
		for(int l=0;l<dimension;l++) {
			// Get the label position
			labIdx=l;
			if(var_idx) {
				labIdx=var_idx->at<int>(l);
			}
			// Fill the position
			labels.at<int>(i,l)=imageLabels[labIdx];
		}
	}

	return labels;
}

void CDadesMuseus::getImageLabels(int imagePos,vector<int> &labels) {
	// Check the image position
	if(imagePos<1) {
		throw CDadesMuseusException("getImageLabels:Image position is out of range");
	}
	if(m_ImageLabels.count(imagePos)<1) {
		throw CDadesMuseusException("getImageLabels:Image not found.");
	}	
	
	// Try to read the image from the local folder	
	labels=m_ImageLabels[imagePos];	
}

void CDadesMuseus::getImageLabels(int imagePos,Mat &labels) {	
	
	// Obtain the labels vector
	vector<int> vec;
	getImageLabels(imagePos,vec);
	
	// Create the Mat object
	// labels=Mat(6,1,CV_32S,vec.data()); // NO FUNCIONA
	//Mat(vec.size(),1,CV_32S,vec.data());
	labels=Mat(vec.size(),1,CV_32S);	
	int i=0;
	for(vector<int>::iterator it = vec.begin(); it != vec.end(); it++)
	{
		labels.at<int>(i)=*it;
		i++;
	}
}

string CDadesMuseus::getLabelName(LabelID labelID) {

	string desc;

	switch(labelID) {
	case LabelID::Author:
		desc="Author";
		break;
	case LabelID::Form:
		desc="Form";
		break;
	case LabelID::School:
		desc="School";
		break;
	case LabelID::Technique:
		desc="Technique";
		break;
	case LabelID::TimeFrame:
		desc="TimeFrame";
		break;
	case LabelID::Type:
		desc="Type";
		break;
	default:
		desc="<Undefined>";
		break;
	}

	return desc;
}

CDadesMuseus::LabelID CDadesMuseus::getLabelID(const unsigned short labelPos) {
	if(labelPos<0 || labelPos>DADES_MUSEUS_NUMLABELS) {
		throw CDadesMuseusException("getLabelID: position out of range.");
	}
	return (LabelID)labelPos;
}

void CDadesMuseus::getProblemDescription(string version,int &numLabels,vector<string> &labelName) {	
	version=DADES_MUSEUS_VERSION;
	numLabels=DADES_MUSEUS_NUMLABELS;
	labelName.clear();	
	for(unsigned short i=0;i<numLabels;i++) {		
		labelName.push_back(getLabelName(getLabelID(i)));
	}
}

string CDadesMuseus::getLabelTranslation(unsigned int labelVal,LabelID labelID) {
	return m_Dictionaries[labelID]->getLabelDesc(labelVal);
}

void CDadesMuseus::getLabelsTranslation(vector<int> labels,vector<string> &translation,vector<LabelID> labIdx) {
	// Check parameter dimensions
	if(labels.size()!=labIdx.size()) {
		throw CDadesMuseusException("getLabelsTranslation: dimensions missmatch.");
	}

	// Translate the vector
	for(unsigned int i=0;i<labels.size();i++) {			
		translation.push_back(getLabelTranslation(labels[i],labIdx[i]));
	}
}

void CDadesMuseus::getLabelsTranslation(Mat labels,vector<string> &translation,vector<LabelID> labIdx) {
	// Check parameter dimensions
	int matSize=labels.size().height*labels.size().width;
	if(matSize!=labIdx.size()) {
		throw CDadesMuseusException("getLabelsTranslation: dimensions missmatch.");
	}

	// Translate the vector
	for(unsigned int i=0;i<labIdx.size();i++) {			
		translation.push_back(getLabelTranslation(labels.at<int>(i),labIdx[i]));
	}
}

void CDadesMuseus::getLabelsTranslation(vector<int> labels,vector<string> &translation,LabelID labIdx) {	
	// Translate the vector
	for(unsigned int i=0;i<labels.size();i++) {			
		translation.push_back(getLabelTranslation(labels[i],labIdx));
	}
}

void CDadesMuseus::getLabelsTranslation(Mat labels,vector<string> &translation,LabelID labIdx) {
	// Check parameter dimensions
	if(labels.size().width>1 && labels.size().height>1) {
		throw CDadesMuseusException("getLabelsTranslation: dimensions missmatch.");
	}

	// Translate the vector
	int maxSize=max(labels.size().height,labels.size().width);
	for(unsigned int i=0;i<maxSize;i++) {		
		translation.push_back(getLabelTranslation(labels.at<int>(i),labIdx));
	}
}

void CDadesMuseus::getLabelsTranslation(vector<int> labels,vector<string> &translation) {
	// Check parameter dimensions
	if(labels.size()!=DADES_MUSEUS_NUMLABELS) {
		throw CDadesMuseusException("getLabelsTranslation: dimensions missmatch.");
	}

	// Create the vector with all labels
	vector<LabelID> labIdx;
	for(unsigned short i=0;i<DADES_MUSEUS_NUMLABELS;i++) {
		labIdx.push_back(getLabelID(i));
	}

	// Delegate the exectution
	getLabelsTranslation(labels,translation,labIdx);
}

void CDadesMuseus::getLabelsTranslation(Mat labels,vector<string> &translation) {
	// Check parameter dimensions
	int maxSize=max(labels.size().height,labels.size().width);
	if(maxSize!=DADES_MUSEUS_NUMLABELS) {
		throw CDadesMuseusException("getLabelsTranslation: dimensions missmatch.");
	}

	// Create the vector with all labels
	vector<LabelID> labIdx;
	for(unsigned short i=0;i<DADES_MUSEUS_NUMLABELS;i++) {
		labIdx.push_back(getLabelID(i));
	}

	// Delegate the exectution
	getLabelsTranslation(labels,translation,labIdx);
}

bool CDadesMuseus::pathExist(string path){ 
	struct stat buf;
	int result;

	// Get information for the path
	result=stat(path.c_str(),&buf);

	// Check the response
	if(result) {		
		switch (errno) {
			case ENOENT:
				return false;           
			default:
				throw CDadesMuseusException("pathExist: Undefined exception.");
		}
	}

	return true;
}

void CDadesMuseus::setLocalStorage(bool value) {
	m_UseLocalStorage=value;
}

void CDadesMuseus::setLocalStoragePath(string path) {
	
	if(!pathExist(path)) {
		throw CDadesMuseusException("setLocalStoragePath: given path does not exist.");
	}
	
	// Store the path
	m_LocalStoragePath=path;
}


Obra CDadesMuseus::getObra(unsigned int obraID) {
	return m_Obres[obraID];
}
 
/// Aquest metode desa la obra pasada per parametres a la llista d'obres amb l'id donat
void CDadesMuseus::setObra(unsigned int obraId, Obra obr) {
	m_Obres[obraId]=obr; 
}
 
    
/// getter: m_num_elements
int CDadesMuseus::getNum_elements(void) {
   return m_num_elements;
}

/// Mostra totes les imatges
void CDadesMuseus::showAllImages(){ 
	try {
		Mat image; Obra obr;
		unsigned int j=0;
		char buffer [33];
		for ( int j = 1; j <=  getNum_elements(); j++) {   
					obr = getObra(j); 
					_itoa_s(obr.getId(), buffer, 10);
					string image_name = "IMAGE";
					image_name.append(buffer);
					namedWindow(image_name);
					image = obr.getImage();
					imshow(image_name, image);
					waitKey(0);
					destroyWindow(image_name);

					// histogram
					obr.showRGB_Histograms(); 
		};

		if (false) {
			
			    // Get the labels and descriptions in OpenCV Mat 
				Mat labelsMat;
				vector<string> labelsDescMat;
				getImageLabels(1,labelsMat);
				getLabelsTranslation(labelsMat,labelsDescMat);

				cout << "---------------------------------------------" << endl;
				cout << "Print the labels obtained as an OpenCV Matrix" << endl;
				cout << "---------------------------------------------" << endl;
				cout << labelsMat << endl;	
				cout << "---------------------------------------------" << endl;
				cout << labelsDescMat << endl;
				cout << "---------------------------------------------" << endl <<endl;
		
				// Repeat using standard vector type
				vector<int> labelsVec;
				vector<string> labelsDescVec;
				getImageLabels(1,labelsVec);
				getLabelsTranslation(labelsVec,labelsDescVec);
			
				cout << "---------------------------------------------" << endl;
				cout << "Print the labels obtained as a Vector" << endl;
				cout << "---------------------------------------------" << endl;
				cout << labelsVec << endl;
				cout << "---------------------------------------------" << endl;
				cout << labelsDescVec << endl;
				cout << "---------------------------------------------" << endl <<endl;

				// Retrieve the matrix of all labels of the fourth first images (1,2,3, and 4)
				Mat sample_idx=(Mat_<int>(1,4) << 1, 2, 3, 4); 
				Mat labelsMatrix=getLabelsMatrix(NULL,&sample_idx);
				cout << "---------------------------------------------" << endl;
				cout << "Print the labels for the fourth first images" << endl;
				cout << "---------------------------------------------" << endl;
				cout << labelsMatrix << endl;
				cout << "---------------------------------------------" << endl;
		
				// Select a subset of labels (0, 3, and 5 -> Author, Type, and TimeFrame 
				Mat var_idx=(Mat_<int>(1,3) << 0, 3, 5); 
				labelsMatrix=getLabelsMatrix(&var_idx,&sample_idx);
				cout << "---------------------------------------------" << endl;
				cout << "Print the labels for the fourth first images" << endl;
				cout << "---------------------------------------------" << endl;
				cout << labelsMatrix << endl;
				cout << "---------------------------------------------" << endl;
		}
	
	} catch(CDadesMuseusException const& dmEx) {
		cout << "An exception occurred:" << endl << dmEx << endl;
	} catch(CProjecteMuseusException const& pmEx) {
		cout << "An exception occurred:" << endl << pmEx << endl;
	}
}

// getter: m_num_elements
int CDadesMuseus::getFeatureType(void){
	return m_feature_type;
}
