#pragma once
#include <string>
#include <map>

using namespace std;

/// <summary>
///  This class implements a dictionar for class labels encoding.
/// </summary>
class CLabelDictionary
{
public:
	/// <summary>
	/// Default constructor.
	/// </summary>
	CLabelDictionary(void);

	/// <summary>
	/// Constructor with a content description
	/// </summary>
	/// <param name="description">Description of the content.</param>	
	CLabelDictionary(string description);

	/// <summary>
	/// Default destructor.
	/// </summary>
	~CLabelDictionary(void);

	/// <summary>
	/// Loads the dictionary data from the given file.
	/// </summary>
	/// <param name="fileName">Filename of the file to be loaded.</param>
	/// <param name="rootPathe">Folder to look for the file. By default is the working path.</param>	
	/// <exception cref="CDadesMuseusException">Thrown when the file cannot be read.</exception>
	void loadFile(string fileName,string rootPath="");

	/// <summary>
	/// Gets the label for the given Id.
	/// </summary>
	/// <param name="id">Numerical value used as class id.</param>	
	/// <returns>The string with the description for the class.</returns>
	string getLabelDesc(int labelId);

	/// <summary>
	/// Gets the id for a given label description.
	/// </summary>
	/// <param name="value">Label description.</param>	
	/// <returns>The integer with the id for the class.</returns>
	int getLabelId(string labelDesc);

private:
	/// <summary>
	/// String with a description for this dictionary
	/// </summary>
	string m_DictionaryDesc;

	/// <summary>
	/// Table with the labels indexed by id
	/// </summary>
	map<int,string> m_IdData;

	/// <summary>
	/// Table with the labels indexed by description
	/// </summary>
	map<string,int> m_DescData;
};

