#include "Obra.h"
#include <string>
#include <map>
#include <opencv2/core/core.hpp>
#include <opencv2/highgui/highgui.hpp> 
#include <opencv2/imgproc/imgproc.hpp> 
#include <string>
#include <iostream>
#include <fstream>
#include <windows.h>
#include <winhttp.h>
#include <stdio.h>
#include <curl/curl.h>
#include <vector>
#include <sys/stat.h>

using namespace cv;
using namespace std;

Obra::Obra(void)
{
}

Obra::Obra(string p_url, int p_id, string p_LocalStoragePath) {

	// global settings
	m_url = p_url; 
	m_id  = p_id;
	m_LocalStoragePath = p_LocalStoragePath; //string m_LocalStoragePath = "../../images/local";
    m_UseLocalStorage=true;

	// obtenim la Obra  
	if (uploadImage()) { 
		// calculem els histogrames
		calc_HSV_histogram();
		calc_RGB_histograms(); 
	}
}

// get the image from the local folder 
// if not found download from internet
bool Obra::uploadImage(){
    Mat image;  
	string fullFileName;
	bool imageLoaded=false;

	// Try to read the image from the local folder 
	if(m_UseLocalStorage) {
		char fileName[1024];

		// Creat the filename
		sprintf_s(fileName,"Image_%07d.jpg",m_id,1023);

		// Create the full filename
		if(m_LocalStoragePath.empty()) {
			fullFileName=string(fileName);
		} else {		
			if(m_LocalStoragePath.at(m_LocalStoragePath.length()-1)=='/' || m_LocalStoragePath.at(m_LocalStoragePath.length()-1)=='\\') {
				fullFileName=m_LocalStoragePath+string(fileName);
			} else {
				fullFileName=m_LocalStoragePath+"/"+string(fileName);			
			}
		}
		// Read the image from the local storage path
		image=imread(fullFileName);
		if(!image.empty()) {
			imageLoaded=true;
		}
	} 

	// Download the image from internet
	if(!imageLoaded) {
		// Download the image
		if(downloadImage(m_url,image)) {
			imageLoaded=true;			
		}
		// Save the image
		if(imageLoaded && m_UseLocalStorage) {
			imwrite(fullFileName,image);
		}
	}
	m_image = image;
	return imageLoaded;
}


bool Obra::downloadImage(string url,Mat &image) {
	CURL *curl=NULL;
	CURLcode res;
	bool retVal=false;	
	vector<char> buffer;

	// Initialize the cURL library object/structure
	curl = curl_easy_init(); 

	if(curl) {
		// Open the URL
        curl_easy_setopt(curl, CURLOPT_URL, url.c_str());

#ifdef _DEBUG
		// Show connection status
        curl_easy_setopt(curl, CURLOPT_VERBOSE, 1); // tell us what is happening
#endif //_DEBUF

        // Tell libcurl where to write the image
        curl_easy_setopt(curl,CURLOPT_WRITEFUNCTION, WriteMemoryCallback);
        curl_easy_setopt(curl,CURLOPT_WRITEDATA, (void *) &buffer);		
		
        // Get the image from the specified URL
        res = curl_easy_perform(curl);

		// decode memory buffer using OpenCV		
		if(CURLE_OK == res) {			
			image = imdecode(Mat(1, buffer.size(), CV_8UC1, buffer.data()), CV_LOAD_IMAGE_UNCHANGED);
			if(!image.empty())
				retVal=true;	
		}

        // always cleanup
        curl_easy_cleanup(curl);        
		buffer.clear();
  } 
  return retVal;
}

size_t Obra::WriteMemoryCallback(void *ptr, size_t size, size_t nmemb, void *data) {
    size_t realsize = size * nmemb;
    std::vector<char> *vec = (vector<char>*)data;
    for(size_t i = 0;i < realsize;i++)		
        vec->push_back(((char*)ptr)[i]);
    return realsize;
}

Mat Obra::getHistogram(void) {
	return m_hist;	
}

Mat Obra::getImage(void) {
	return m_image;	
}

void Obra::setFold(int fold) { 
    m_fold = fold; 
}

int Obra::getFold(void){ 
	return m_fold;
}

void Obra::setIsTrainning_set(bool isTrainning_set) {
     m_isTrainning_set = isTrainning_set ;
}

bool Obra::getIsTrainning_set(void) {
	return m_isTrainning_set;
}

void Obra::setClass(int aClassID)  {
     m_classID = aClassID ;
}

int Obra::getClass(void){
	return m_classID;
}


Obra::~Obra(void)
{
}


// show histogram 
void Obra::showRGB_Histograms(void) {
     
	Mat histImg; 
	
	histImg = imHist(g_hist,3,3,1);

	string image_name = "Green "; 
	image_name.append(m_url);
	namedWindow(image_name, CV_WINDOW_AUTOSIZE );
	imshow(image_name, histImg ); 
	waitKey(0); 
	destroyWindow(image_name); 
	
	
	histImg = imHist(r_hist,3,3,2);
	image_name = "Red ";
	image_name.append(m_url);
	namedWindow(image_name, CV_WINDOW_AUTOSIZE );
	imshow(image_name, histImg ); 
	waitKey(0); 
	destroyWindow(image_name); 
    //imshow("Red",histImg);

	histImg = imHist(b_hist,3,3,0); 
	image_name = "Blue ";
	image_name.append(m_url);
	namedWindow(image_name, CV_WINDOW_AUTOSIZE );
	imshow(image_name, histImg ); 
	waitKey(0); 
	destroyWindow(image_name); 
	//imshow("Blue",histImg);

	 

	/**
	/// Establish the number of bins 
	int histSize = 255;
	
	// Draw the histograms for R, G and B
	int hist_w = 400; int hist_h = 400;
	int bin_w = cvRound( (double) hist_w/histSize );

	Mat histImage( hist_w, hist_h, CV_8UC3, Scalar( 0,0,0) ); 

	/// Draw for each channel 
	for( int i = 1; i < histSize; i++ )
	{ 
			line( histImage, Point( bin_w*(i-1), hist_h - cvRound(r_hist.at<float>(i-1)) ) , 
  					 Point( bin_w*(i), hist_h - cvRound(r_hist.at<float>(i)) ), 
						 Scalar( 0, 0, 255), 2, 8, 0  ); 
			
			line( histImage, Point( bin_w*(i-1), hist_h - cvRound(g_hist.at<float>(i-1)) ) , 
					 Point( bin_w*(i), hist_h - cvRound(g_hist.at<float>(i)) ), 
						 Scalar( 0, 255, 0), 2, 8, 0  ); 
			line( histImage, Point( bin_w*(i-1), hist_h - cvRound(b_hist.at<float>(i-1)) ) , 
					 Point( bin_w*(i), hist_h - cvRound(b_hist.at<float>(i)) ), 
						 Scalar( 255, 0, 0), 2, 8, 0  );        
	}

	/// Display  
	string image_name = "IMAGE ";
	image_name.append(m_url);
	namedWindow(image_name, CV_WINDOW_AUTOSIZE );
	imshow(image_name, histImage ); 
	waitKey(0); 
	destroyWindow(image_name);

	**/
}

void Obra::calc_HSV_histogram(void){
    // calculem el histograma
	Mat hsv, H; MatND tempH;
	cvtColor(m_image, hsv, CV_BGR2HSV);
	int planes[]= { 0, 1};
	int hsize[] = { 32, 32};
	calcHist(&hsv, 1, planes, Mat(), tempH, 2, hsize, 0);
	H = tempH;
	m_hist = H;   
}

void Obra::calc_RGB_histograms(void){
	Mat src, dst;

    /// Load image
    src = m_image; 

	/// Separate the image in 3 places ( R, G and B )
	vector<Mat> rgb_planes;
	split( src, rgb_planes ); 

	/// Establish the number of bins 
	int histSize = 256;

	/// Set the ranges ( for R,G,B) )
	float range[] = { 0, 255 } ; 
	const float* histRange = { range };  

	/// Compute the histograms
	bool uniform = true; bool accumulate = false; 
	calcHist( &rgb_planes[0], 1, 0, Mat(), b_hist, 1, &histSize, &histRange, uniform, accumulate );
	calcHist( &rgb_planes[1], 1, 0, Mat(), g_hist, 1, &histSize, &histRange, uniform, accumulate );
	calcHist( &rgb_planes[2], 1, 0, Mat(), r_hist, 1, &histSize, &histRange, uniform, accumulate ); 
	 
	// Normalize the result to [ 0, 400] 
	int hist_w = 400; 
	normalize(r_hist, r_hist, 0, hist_w, NORM_MINMAX, -1, Mat() );
	normalize(g_hist, g_hist, 0, hist_w, NORM_MINMAX, -1, Mat() );
	normalize(b_hist, b_hist, 0, hist_w, NORM_MINMAX, -1, Mat() ); 
}



double Obra::distancia(Obra obr) {
	double resultat =   compareHist(r_hist, obr.getHist_r(), CV_COMP_CHISQR       ) +
						compareHist(g_hist, obr.getHist_g(), CV_COMP_CHISQR       ) +
						compareHist(b_hist, obr.getHist_b(), CV_COMP_CHISQR       ) ; 

	// show histogramS
	//obr.showRGB_Histograms();
	//showRGB_Histograms();

	//resultat = resultat / 3;
	if (false) {
		cout << "---------------- distiancia entre histogrames -----------------------------" << endl;
		cout << m_id << endl;
		cout << obr.getId() << endl;
		cout << resultat << endl;
	} 
	return resultat;
}

Mat Obra::getHist_r(void){return r_hist;}
Mat Obra::getHist_g(void){return g_hist;}
Mat Obra::getHist_b(void){return b_hist;}
int Obra::getId(void){return m_id;}


Mat  Obra::imHist(Mat hist, float scaleX, float scaleY, int color){
  double maxVal=0;
  minMaxLoc(hist, 0, &maxVal, 0, 0);
  int rows = 64; //default height size
  int cols = hist.rows; //get the width size from the histogram
  Mat histImg = Mat::zeros(rows*scaleX, cols*scaleY, CV_8UC3);
  //for each bin
  for(int i=0;i<cols-1;i++) {
    float histValue = hist.at<float>(i,0);
    float nextValue = hist.at<float>(i+1,0);
    Point pt1 = Point(i*scaleX, rows*scaleY);
    Point pt2 = Point(i*scaleX+scaleX, rows*scaleY);
    Point pt3 = Point(i*scaleX+scaleX, (rows-nextValue*rows/maxVal)*scaleY);
    Point pt4 = Point(i*scaleX, (rows-nextValue*rows/maxVal)*scaleY);

    int numPts = 5;
    Point pts[] = {pt1, pt2, pt3, pt4, pt1};

	switch (color) {
		case (0) :  
			fillConvexPoly(histImg, pts, numPts, Scalar(255,0,0));
			break; 
		case (1) :  
			fillConvexPoly(histImg, pts, numPts, Scalar(0,255,0));
			break; 
		case (2) :  
			fillConvexPoly(histImg, pts, numPts, Scalar(0,0,255));
			break;
	}
		 
  } 
  return histImg;
}