#pragma once
#include <string>
#include <iostream>
#include <map>
#include <opencv2/core/core.hpp>
#include <opencv2/highgui/highgui.hpp>
#include <opencv2/imgproc/imgproc.hpp> 
#include "Obra.h"

using namespace cv;
using namespace std;

class Obra
{
public:
	/// <summary>
	/// Default constructor.
	/// </summary>
	Obra(void); 

	/// <summary>
	/// Constructor with parms
	/// </summary> 
	Obra(string p_url, int p_id, string p_LocalStoragePath);
	   
	/// <summary>
	/// Default destructor.
	/// </summary>  
	~Obra(void);

	// show histogram 
	void showRGB_Histograms(void);
	double distancia(Obra obr); 

	// setters
	void setClass(int aClassID);
	void setFold(int fold);
	void setIsTrainning_set(bool isTrainning_set);

	// getters
	Mat getHistogram(void); 
	Mat getImage(void);
	int getFold(void);
	int getClass(void);
	int getId(void);
	bool getIsTrainning_set(void);
	Mat getHist_r(void);
	Mat getHist_g(void);
	Mat getHist_b(void);

	
private: 
	
	/// <summary>
	/// URL from where the image was donloaded
	/// </summary>
	string m_url; 

	/// <summary>
	/// Path where images are stored in case local storage is activated.
	/// </summary>
	string m_LocalStoragePath;
	
	/// <summary>
	/// local store activation flag
	/// </summary>
	bool m_UseLocalStorage;

	int m_id; 
	int m_fold; 
	int m_classID; 
	Mat m_image; 
	Mat m_hist; 
	bool m_isTrainning_set;
	Mat r_hist, g_hist, b_hist;

	static size_t WriteMemoryCallback(void *ptr, size_t size, size_t nmemb, void *data); 
	
	bool downloadImage(string url,Mat &image); 

	bool uploadImage();

	void calc_HSV_histogram(void);
	void calc_RGB_histograms(void);
	Mat  imHist(Mat hist, float scaleX=1, float scaleY=1, int color=0);
	
};

