/*! \mainpage ProjecteMuseus Project
 *
 * \section intro_sec Introduction
 *
 * This project contains the basic classes and libraries to access the art images database. The actual version
 * contains a Microsoft Visual Studio 10 project with all the libraries. A demo code is provided in order to ilustrate
 * some main classes.
 *
 * \section Documentation
 *
 * All the code is documented using XML tags, readable by Doxygen and Windows Help generators. A folder named documentation
 * contains all the documentation in diferent formats.
 *
 *
 * \section Contact
 *	This project is in development. Please, for any suggestion or further information, contact by mail:
 *		Xavier Bar  xbaro (a) uoc (dot) edu
 *		 
 */

#include "stdafx.h"
#include "DadesMuseus.h"
#include "Classificador.h"
#include <opencv2/core/core.hpp>
#include <opencv2/highgui/highgui.hpp> 
#include <string>
#include <iostream>
#include <atlstr.h>
#include <windows.h>

using namespace cv;
using namespace std;

Mat imHist(Mat hist, float scaleX=1, float scaleY=1, int color=0){
  double maxVal=0;
  minMaxLoc(hist, 0, &maxVal, 0, 0);
  int rows = 64; //default height size
  int cols = hist.rows; //get the width size from the histogram
  Mat histImg = Mat::zeros(rows*scaleX, cols*scaleY, CV_8UC3);
  //for each bin
  for(int i=0;i<cols-1;i++) {
    float histValue = hist.at<float>(i,0);
    float nextValue = hist.at<float>(i+1,0);
    Point pt1 = Point(i*scaleX, rows*scaleY);
    Point pt2 = Point(i*scaleX+scaleX, rows*scaleY);
    Point pt3 = Point(i*scaleX+scaleX, (rows-nextValue*rows/maxVal)*scaleY);
    Point pt4 = Point(i*scaleX, (rows-nextValue*rows/maxVal)*scaleY);

    int numPts = 5;
    Point pts[] = {pt1, pt2, pt3, pt4, pt1};

	switch (color) {
		case (0) :  
			fillConvexPoly(histImg, pts, numPts, Scalar(255,0,0));
			break; 
		case (1) :  
			fillConvexPoly(histImg, pts, numPts, Scalar(0,255,0));
			break; 
		case (2) :  
			fillConvexPoly(histImg, pts, numPts, Scalar(0,0,255));
			break;
	}
		 
  } 
  return histImg;
}


//int _tmain(int argc, _TCHAR* argv[])
	int main(int argc, char *argv[], char *envp[])
{
	bool test = true; 
	//check num of arguments
	if (argc != 7) {
		
		if (!test) { 
		cout << "\nNombre incorrecte de parametres. Els parametres son:" << endl;
			cout << "\n1: Directory arrel on es troba el conjunt d'imatges a analitzar." << endl;
			cout << "\n2: Nombre de folds." << endl;
			cout << "\n3: Nombre d'elements de cada classe a cada fold." << endl;
			cout << "\n4: Tipus d'atribut. Disponibles:" << endl; 
			cout << "\n                    1: autor" << endl;
			cout << "\n                    2: tcnica" << endl;
			cout << "\n                    3: format de lobra" << endl;
			cout << "\n                    4: tipus dobra dart" << endl;
			cout << "\n                    5: escola" << endl;
			cout << "\n                    6: perode" << endl;
			cout << "\n5: Activar mode debug?: 1 --> si" << endl;  
			cout << "\n5: Num veins KNN. K? " << endl; 
		} else {  

		   Obra obr1("http://www.wga.hu/art/v/velazque/01/0110vela.jpg", 11,"../images_run6/local");
		   Obra obr2("http://www.livingspain.es/wp-content/uploads/2009/08/Mae-West.jpg", 38, "../images_run6/local"); 
		   obr1.showRGB_Histograms();
		   obr2.showRGB_Histograms();

           /**
		   // Check distancia 
		   Obra obr1("http://www.wga.hu/art/a/abaquesn/floor2.jpg", 3,"../images_run1/local");
		   Obra obr2("http://www.wga.hu/art/a/abaquesn/theflood.jpg", 4, "../images_run1/local"); 
		   obr1.showRGB_Histograms();
		   obr2.showRGB_Histograms();
		   int dist1 = obr1.distancia(obr2);

		    
		   // Check distancia 
		   Obra obr3("http://www.wga.hu/art/a/abadia/michael.jpg", 3, "../images_test1/local");
		   Obra obr4("http://www.wga.hu/art/a/abadia/michael.jpg", 4, "../images_test1/local"); 
		   int dist2 = obr3.distancia(obr4);

		   // Check distancia 
		   Obra obr5("http://www.wga.hu/art/a/abaquesn/albarell.jpg", 5, "../images_test1/local");
		   Obra obr6("http://www.wga.hu/art/a/abaquesn/albarell.jpg", 6, "../images_test1/local"); 
		   int dist3 = obr5.distancia(obr6);

		   // Check distancia 
		   Obra obr7("http://www.wga.hu/art/a/abbate/chimney1.jpg", 7, "../images_test1/local");
		   Obra obr8("http://www.wga.hu/art/a/abbate/chimney1.jpg", 8, "../images_test1/local"); 
		   int dist4 = obr7.distancia(obr8);

		   Obra obr9("http://www.wga.hu/art/a/abildgaa/culmingh.jpg", 9, "../images_test1/local");
		   Obra obr10("http://www.wga.hu/art/a/abildgaa/culmingh.jpg", 10, "../images_test1/local"); 
		   int dist5 = obr9.distancia(obr10);

	       */
		
		   Classificador clf1("../images_run1/", 4, 1, 3, 3,true);  
		   clf1.run();

		   cout << "\nPrem una tecla per continuar..." << endl;
		   getchar();  

		   return 0;

		}

		return 1;
	} else {
	
		try { 
			   char * _path = new char [100];			
			   _path  = (char *) argv[1];     
			   char * _folds = new char [1];			
			  _folds = (char *) argv[2];
			   char * _class_elements = new char [1];	
			   _class_elements = (char *) argv[3];
			   char * _atribut = new char [1];			
			   _atribut = (char *) argv[4];
			   char * _debug = new char [1];			
			   _debug = (char *) argv[5];
			   bool _debug_bool = false;  
			   if (atoi(_debug) == 1) {
				  _debug_bool = true; 
			   }
			   char * _k = new char [1];			
			   _k = (char *) argv[6];

			   Classificador clf1(_path, atoi(_folds), atoi(_class_elements), atoi(_atribut), atoi(_k), _debug_bool);  
			   clf1.run();

		   
				cout << "\nPrem una tecla per continuar..." << endl;
				getchar();

				return 0;

		} catch(CDadesMuseusException const& dmEx) {
				cout << "An exception occurred:" << endl << dmEx << endl;
		} catch(CProjecteMuseusException const& pmEx) {
				cout << "An exception occurred:" << endl << pmEx << endl;
		}

	} 




			// Case 1: 3 FOLDS amb un element de cada classe al fold 
			//         atribut = autor 
			//Classificador clf1("../images_case1/", 3, 1, 1, true);  
			//clf1.run();


		/**

		/// Load image
		Mat src = imread("C:\\lena.jpg",CV_LOAD_IMAGE_COLOR );
		Mat histImg;

		/// Separate the image in 3 places ( R, G and B )
		vector<Mat> rgb_planes;
		split( src, rgb_planes ); 

		/// Establish the number of bins 
		int histSize = 256;

		/// Set the ranges ( for R,G,B) )
		float range[] = { 0, 255 } ; 
		const float* histRange = { range };
	
		bool uniform = true; bool accumulate = false;

		MatND r_hist, g_hist, b_hist;

		/// Compute the histograms:
		calcHist( &rgb_planes[0], 1, 0, Mat(), b_hist, 1, &histSize, &histRange, uniform, accumulate );
		int hist_w = 400;
		normalize(b_hist, b_hist, 0, hist_w, NORM_MINMAX, -1, Mat() );
		histImg = imHist(b_hist,3,3,0);
		imshow("Blue",histImg);
	 
		calcHist( &rgb_planes[1], 1, 0, Mat(), g_hist, 1, &histSize, &histRange, uniform, accumulate );
		normalize(g_hist, g_hist, 0, hist_w, NORM_MINMAX, -1, Mat() );
		histImg = imHist(g_hist,3,3,1);
		imshow("Green",histImg); 


		calcHist( &rgb_planes[2], 1, 0, Mat(), r_hist, 1, &histSize, &histRange, uniform, accumulate ); 
		normalize(r_hist, r_hist, 0, hist_w, NORM_MINMAX, -1, Mat() );
		histImg = imHist(r_hist,3,3,2);
		imshow("Red",histImg);

	 
		// Draw the histograms for R, G and B
		hist_w = 400; int hist_h = 400;
		int bin_w = cvRound( (double) hist_w/histSize );
		Mat histImage( hist_w, hist_h, CV_8UC3, Scalar( 0,0,0) );

		/// Normalize the result to [ 0, histImage.rows ]
		normalize(r_hist, r_hist, 0, hist_w, NORM_MINMAX, -1, Mat() );
		normalize(g_hist, g_hist, 0, hist_w, NORM_MINMAX, -1, Mat() );
		normalize(b_hist, b_hist, 0, hist_w, NORM_MINMAX, -1, Mat() );

		for( int i = 1; i < histSize; i++ )
		{ 
				line( histImage, Point( bin_w*(i-1), hist_h - cvRound(r_hist.at<float>(i-1)) ) , 
  						 Point( bin_w*(i), hist_h - cvRound(r_hist.at<float>(i)) ), 
							 Scalar( 0, 0, 255), 2, 8, 0  ); 
			
				line( histImage, Point( bin_w*(i-1), hist_h - cvRound(g_hist.at<float>(i-1)) ) , 
						 Point( bin_w*(i), hist_h - cvRound(g_hist.at<float>(i)) ), 
							 Scalar( 0, 255, 0), 2, 8, 0  ); 
				line( histImage, Point( bin_w*(i-1), hist_h - cvRound(b_hist.at<float>(i-1)) ) , 
						 Point( bin_w*(i), hist_h - cvRound(b_hist.at<float>(i)) ), 
							 Scalar( 255, 0, 0), 2, 8, 0  );        
		}

		/// Display   
		namedWindow("IMAGE ", CV_WINDOW_AUTOSIZE );
		imshow("IMAGE ", histImage ); 
		waitKey(0); 
		destroyWindow("IMAGE ");


		**/






		 
			// Case 1: 3 FOLDS amb un element de cada classe al fold 
			//         atribut = autor 
			//Classificador clf1("../images_case1/", 3, 1, 1, true);  
			//clf1.run();

			// Case 2: 4 FOLDS amb un element de cada classe al fold 
			//         atribut = autor 
			//Classificador clf2("../images_case2/", 4, 1, 1);  

			// Case 3: 4 FOLDS amb un element de cada classe al fold 
			//         atribut = form 
			//Classificador clf3("../images_case3/", 4, 1, 3);
			//clf3.showAllImages();

			// Case 4: 7 FOLDS amb un element de cada classe al fold 
			//         atribut = type 
			//Classificador clf4("../images_case3/", 7, 1, 4);
		 
}
